@@ -6,7 +6,7 @@ import os, sys, time
 import warnings
 import ast  # ast.literal_eval is safe eval
 import numpy as np
-from collections import defaultdict  # since Python 2.5
+import collections  # defaultdict since Python 2.5, OrderedDict since Python 3.1
 from .python3for2 import abc, range
 del absolute_import, division, print_function  #, unicode_literals, with_statement
 
@@ -331,20 +331,48 @@ def num2str(val, significant_digits=2, force_rounding=False,
     return s_ret
 
 # todo: this should rather be a class instance
+warnings_counter = collections.defaultdict(int)
 def print_warning(msg, method_name=None, class_name=None, iteration=None,
-                   verbose=None, maxwarns=None):
-    """Poor man's maxwarns: warn only if ``iteration<=maxwarns``"""
+                   verbose=None, maxwarns=None, **kwargs_for_warn):
+    """Poor man's maxwarns: msg must match exactly"""
     if verbose is None:
         verbose = global_verbosity
-    if maxwarns is not None and iteration is None:
-        raise ValueError('iteration must be given to activate maxwarns')
-    if verbose >= -2 and (iteration is None or maxwarns is None or
-                            iteration <= maxwarns):
-        warnings.warn(msg + ' (' +
-              ('class=%s ' % str(class_name) if class_name else '') +
-              ('method=%s ' % str(method_name) if method_name else '') +
-              ('iteration=%s' % str(iteration) if iteration else '') +
-              ')')
+    if maxwarns is not None:  # we could do the counting irrespectively?
+        warnings_counter[msg] += 1
+        if warnings_counter[msg] > maxwarns:
+            return
+        if warnings_counter[msg] == maxwarns:
+            msg += ' (further warnings will be suppressed)'
+    if verbose >= -2:
+        warnings.warn(msg + ' (time={}'.format(time.asctime()[4:]) +
+              (' class=%s' % str(class_name) if class_name else '') +
+              (' method=%s' % str(method_name) if method_name else '') +
+              (' iteration=%s' % str(iteration) if iteration else '') +
+              ')', **kwargs_for_warn)
+def format_warning(msg, method_name=None, class_name=None, iteration=None,
+                   maxwarns=None):
+    """Poor man's maxwarns: msg must match exactly.
+
+    Copy-paste of `print_warning` to get better location information than
+    `print_warning`. Calling `warnings.warn` here makes the warning
+    location information meaningless, hence we format only a string here.
+    Usage could be like::
+
+        m = utils.format_warning(some_message
+                ); m and warnings.warn(m)
+
+    """
+    if maxwarns is not None:  # we could do the counting irrespectively?
+        warnings_counter[msg] += 1
+        if warnings_counter[msg] > maxwarns:
+            return ''
+        if warnings_counter[msg] == maxwarns:
+            msg += ' (further warnings will be suppressed)'
+    return (msg + ' (time={}'.format(time.asctime()[4:]) +
+            (' class=%s' % str(class_name) if class_name else '') +
+            (' method=%s' % str(method_name) if method_name else '') +
+            (' iteration=%s' % str(iteration) if iteration else '') +
+            ')')
 def print_message(msg, method_name=None, class_name=None, iteration=None,
                    verbose=None):
     if verbose is None:
@@ -355,6 +383,19 @@ def print_message(msg, method_name=None, class_name=None, iteration=None,
               (', method=' + str(method_name) if method_name else '') +
               (', iteration=' + str(iteration) if iteration is not None else '') +
               '): ', msg)
+def format_message(msg, es=None, spaces=6):
+    """put line breaks and trailing white spaces"""
+    if es:
+        msg = msg + ' [iteration={0}]'.format(es.countiter)
+    j = 0
+    s = "\n\n" + spaces * " "
+    for i, c in enumerate(msg):
+        s = s + c
+        if c == '\n' or i > j + 70 and c in [' ']:
+            s = s + '\n' + spaces * ' '
+            j = i
+    s = s + '\n'
+    return s
 
 def set_attributes_from_dict(self, dict_, initial_params_dict_name=None):
     """assign, for example, all arguments given to an ``__init__``
@@ -467,8 +508,12 @@ class DerivedDictBase(abc.MutableMapping):
         # abc.MutableMapping.__init__(self)
         super(DerivedDictBase, self).__init__()
         # super(SolutionDict, self).__init__()  # the same
-        self.data = dict()
-        self.data.update(dict(*args, **kwargs))
+        try:
+            self.data = collections.OrderedDict()
+            self.data.update(collections.OrderedDict(*args, **kwargs))
+        except Exception:
+            self.data = dict()
+            self.data.update(dict(*args, **kwargs))
     def __len__(self):
         return len(self.data)
     def __contains__(self, key):
@@ -491,18 +536,18 @@ class SolutionDict(DerivedDictBase):
     previously inserted same solutions is provided. Each entry is meant
     to store additional information related to the solution.
 
-        >>> import cma.utilities.utils as utils, numpy as np
-        >>> d = utils.SolutionDict()
-        >>> x = np.array([1,2,4])
-        >>> d[x] = {'f': sum(x**2), 'iteration': 1}
-        >>> assert d[x]['iteration'] == 1
-        >>> assert d.get(x) == (d[x] if d.key(x) in d.keys() else None)
-        >>> y = [1,2,4]
-        >>> d[y] = {'f': sum([n ** 2 for n in y]), 'iteration': 1}
-        >>> assert d[y]['iteration'] == 1
-        >>> assert d.get(y) == (d[y] if d.key(y) in d.keys() else None)
-        >>> d[2] = 3
-        >>> assert d[2] == 3
+    >>> import cma.utilities.utils as utils, numpy as np
+    >>> d = utils.SolutionDict()
+    >>> x = np.array([1,2,4])
+    >>> d[x] = {'f': sum(x**2), 'iteration': 1}
+    >>> assert d[x]['iteration'] == 1
+    >>> assert d.get(x) == (d[x] if d.key(x) in d.keys() else None)
+    >>> y = [1,2,4]
+    >>> d[y] = {'f': sum([n ** 2 for n in y]), 'iteration': 1}
+    >>> assert d[y]['iteration'] == 1
+    >>> assert d.get(y) == (d[y] if d.key(y) in d.keys() else None)
+    >>> d[2] = 3
+    >>> assert d[2] == 3
 
     TODO: data_with_same_key behaves like a stack (see setitem and
     delitem), but rather should behave like a queue?! A queue is less
@@ -523,9 +568,9 @@ class SolutionDict(DerivedDictBase):
             # Do not hash key again
             return x
         elif isinstance(x, np.ndarray):
-            try: 
+            try:
                 return hash(x.tobytes())
-            except AttributeError: 
+            except AttributeError:
                 if x.size < 1e4:  # based on timing results
                     return hash(tuple(x))
                 else:
@@ -533,9 +578,11 @@ class SolutionDict(DerivedDictBase):
         else:
             try:
                 return hash(x)
-            except TypeError: 
+            except TypeError:
                 # Data type must be immutable, transform into tuple first
                 return hash(tuple(x))
+    def __contains__(self, key):
+        return super(SolutionDict, self).__contains__(self.key(key))
     def __setitem__(self, key, value):
         """define ``self[key] = value``"""
         key = self.key(key)
@@ -557,20 +604,36 @@ class SolutionDict(DerivedDictBase):
                 self.data[key] = self.data_with_same_key[key].pop(-1)
         elif key in self.data:
             del self.data[key]
-    def truncate(self, max_len, min_iter):
-        """delete old entries to prevent bloat"""
-        if len(self) > max_len:
+    def truncate(self, max_len, min_iter=None):
+        """truncate to ``max_len/2`` when ``len(self) > max_len``.
+
+        Only truncate entries with ``'iteration'`` key smaller than
+        `min_iter` if given.
+        """
+        if len(self) <= max_len:
+            return
+        if min_iter is None:  # new with OrderedDict
+            l = max((0, max_len / 2))
+            while len(self) > l:
+                self.data.popitem(last=False)
+        else:  # previous code
             for k in list(self.keys()):
                 if self[k]['iteration'] < min_iter:
                     del self[k]
                     # deletes one item with k as key, better delete all?
+                if len(self) < max_len / 2:  # new code
+                    break
 
-class DataDict(defaultdict):
+class DataDict(collections.defaultdict):
     """a dictionary of lists (of data)"""
     def __init__(self, filename='_data.py'):
         self.filename = filename
-        defaultdict.__init__(self, list)
-        self.load()
+        collections.defaultdict.__init__(self, list)
+        if os.path.exists(self.filename):
+            self.load()
+        else:
+            print('creating file {0}'.format(self.filename))
+            self.save()
 
     def load(self):
         """element-wise append/merge data of loaded `dict` to self,
@@ -588,7 +651,7 @@ class DataDict(defaultdict):
     def update(self, dict_):
         """append data of entries in `dict_` to entries in self"""
         for k in dict_:
-            self[k] += dd[k]  # self is a dict of lists
+            self[k] += dict_[k]  # self is a dict of lists
         return self
 
     def save(self):
@@ -964,3 +1027,95 @@ class ListOfCallables(list):
             assert len(self) == len(res) == 1
             return res[0]  # for backwards compatibility when a single callable is used
         return res
+
+class ShowInFolder(object):  # was ShowInline
+    """callable instance to save and show figures from `matplotlib`.
+
+    Saves figures to a folder ``'figs-...'`` with incremental filenames
+    allowing to conveniently view and compare these figures.
+
+    Each call of the instance saves the current `matplotlib` figure and
+    switches the focus to the save folder by calling the `open_path`
+    method.
+
+    Example::
+
+        >> import cma, matplotlib.pyplot as plt
+        >> show = cma.utilities.utils.ShowInline('01')  # use './figs-01' folder
+        >>
+        >> plt.plot([1,2,3])
+        >> show()  # save current plot with a unique name in figs-01 folder
+        >>         # and switch focus to the folder
+
+    Under macOS, `open_path` invokes Finder where the figures can be
+    conveniently explored and compared using the space and up/down keys.
+    """
+    folder_prefix = 'figs-'
+    def __init__(self, name, type='pdf', dpi=400, **kwargs):
+        """save figures to folder ``'figs-' + name``.
+
+        pdf are smaller than png and not pixeliated. The type can be
+        changed at any time be reassigning the `type` attribute.
+
+        `dpi` and `kwargs` are passed to `plt.savefig`.
+        """
+        self.path =  ShowInFolder.folder_prefix + name
+        """output folder path"""
+        self.type = type
+        self.format_string = "{0:04d}."
+        """filename format string"""
+        self.kwargs = kwargs
+        """kwargs to ``plt.savefig``"""
+        self.kwargs['dpi'] = dpi
+        self.number = 0
+        """number of saved figures or id of last saved figure"""
+        if os.path.exists(self.path):
+            ls = sorted(n for n in os.listdir(self.path)
+                    if n[0] in [str(i) for i in range(10)] and n.endswith('.' + type))
+            if ls:
+                self.number = int(ls[-1].split('.' + type)[0])
+
+    def __call__(self,
+                 keep=True, show_in_notebook=False,
+                 show_in_folder=True, save=True,
+                 width_save=None, width_show=None,
+                 name=None):
+        """save current figure and show folder and/or image in notebook.
+
+        Uses `IPython.display.Image` to show the image in the notebook.
+        """
+        if save:
+            from matplotlib import pyplot as plt
+            if not os.path.exists(self.path):
+                if self.number != 0:
+                    warnings.warn(self.path + " not found"
+                                  " even though number={0} (which should ever happen)."
+                                  .format(self.number))
+                os.mkdir(self.path)
+            if name is None:
+                self.number += 1
+                name = self.format_string.format(self.number) + self.type
+            name = os.path.join(self.path, name)
+            if width_save:
+                old_size = plt.gcf().get_size_inches()  # keep aspect ratio
+                plt.gcf().set_size_inches(width_save,
+                                          width_save * old_size[1] / old_size[0])
+            # plt.tight_layout()
+            plt.savefig(name, bbox_inches='tight', **self.kwargs)
+            self._last_saved = name
+            keep or plt.close()
+        show_in_folder and self.open_path()
+        if self.number and show_in_notebook:
+            self.show_in_notebook(width_show)
+        return self
+
+    def show_in_notebook(self, width=None, **kwargs_show):
+        """display last saved figure in notebook (doesn't work with pdf)"""
+        try:
+            import IPython.display
+            return IPython.display.Image(self._last_saved, width=width, **kwargs_show)
+        except:
+            print("{0} could not be displayed".format(self._last_saved))
+
+    def open_path(self):
+        os.system('open ' + self.path)
